﻿namespace HeuristicLab.Problems.ProgramSynthesis.Base.Erc {
  using System;
  using System.Collections.Generic;
  using System.Linq;
  using Base.Erc.Interfaces;
  using Common;
  using Core;
  using Data;
  using Parameters;
  using Persistence.Default.CompositeSerializers.Storable;

  [StorableClass]
  public class CharErcOptions : ErcOption, IReadOnlyCharErcOptions {
    private const string ConstantsParameterName = "Constants";
    private const string RangeParameterName = "Range";

    public CharErcOptions() : this('A', 'z') {
      IsEnabled = false;
    }

    public CharErcOptions(int start, int end) : this(start, end, new char[0]) {
    }

    public CharErcOptions(char[] constants) : this(default(int), default(int), constants) {
    }

    public CharErcOptions(int start, int end, char[] constants) {
      if (start > end) {
        throw new InvalidOperationException("start > end");
      }

      IsEnabled = true;
      Parameters.Add(new ValueParameter<StringValue>(ConstantsParameterName, new StringValue(new string(constants))));
      Parameters.Add(new FixedValueParameter<IntRange>(RangeParameterName, new IntRange(start, end)));
    }

    [StorableConstructor]
    private CharErcOptions(bool deserializing) : base(deserializing) { }

    public CharErcOptions(CharErcOptions origin, Cloner cloner) : base(origin, cloner) { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new CharErcOptions(this, cloner);
    }

    public IValueParameter<StringValue> ConstantsParameter
    {
      get { return (IValueParameter<StringValue>)Parameters[ConstantsParameterName]; }
    }

    public IReadOnlyList<char> Constants
    {
      get { return ConstantsParameter.Value.Value.ToCharArray(); }
      set { ConstantsParameter.Value.Value = new string(value.ToArray()); }
    }

    public IValueParameter<IntRange> RangeParameter
    {
      get { return (IValueParameter<IntRange>)Parameters[RangeParameterName]; }
    }

    public int Start
    {
      get { return RangeParameter.Value.Start; }
      set { RangeParameter.Value.Start = value; }
    }

    public int End
    {
      get { return RangeParameter.Value.End; }
      set { RangeParameter.Value.End = value; }
    }
  }
}
