﻿namespace HeuristicLab.Problems.ProgramSynthesis.Base.Erc {
  using HeuristicLab.Common;
  using HeuristicLab.Core;
  using HeuristicLab.Data;
  using HeuristicLab.Parameters;
  using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
  using HeuristicLab.Problems.ProgramSynthesis.Base.Erc.Interfaces;

  [StorableClass]
  public class ErcOptions : ParameterizedNamedItem, IReadOnlyErcOptions {
    public const string ProbabilityParameterName = "Probability";
    public const string IntegerParameterName = "Integer";
    public const string FloatParameterName = "Float";
    public const string CharParameterName = "Char";
    public const string BooleanParameterName = "Boolean";
    public const string StringParameterName = "String";
    public const string NameParameterName = "Name";
    public const string IntegerVectorParameterName = "IntegerVector";
    public const string FloatVectorParameterName = "FloatVector";
    public const string StringVectorParameterName = "StringVector";

    public ErcOptions() {
      Name = "ERC Options";

      Parameters.Add(new FixedValueParameter<PercentValue>(ProbabilityParameterName));
      Parameters.Add(new FixedValueParameter<IntegerErcOptions>(IntegerParameterName));
      Parameters.Add(new FixedValueParameter<FloatErcOptions>(FloatParameterName));
      Parameters.Add(new FixedValueParameter<CharErcOptions>(CharParameterName));
      Parameters.Add(new FixedValueParameter<BooleanErcOptions>(BooleanParameterName));
      Parameters.Add(new FixedValueParameter<StringErcOptions>(StringParameterName));
      Parameters.Add(new FixedValueParameter<NameErcOptions>(NameParameterName));
      Parameters.Add(new FixedValueParameter<IntegerVectorErcOptions>(IntegerVectorParameterName));
      Parameters.Add(new FixedValueParameter<FloatVectorErcOptions>(FloatVectorParameterName));
      Parameters.Add(new FixedValueParameter<StringVectorErcOptions>(StringVectorParameterName));
    }

    [StorableConstructor]
    public ErcOptions(bool deserializing) : base(deserializing) { }

    public ErcOptions(ErcOptions origin, Cloner cloner) : base(origin, cloner) { }

    public IValueParameter<PercentValue> ErcProbabilityParameter
    {
      get { return (IValueParameter<PercentValue>)Parameters[ProbabilityParameterName]; }
    }

    public double ErcProbability
    {
      get { return ErcProbabilityParameter.Value.Value; }
      set { ErcProbabilityParameter.Value.Value = value; }
    }

    public IValueParameter<IntegerErcOptions> IntegerErcOptionsParameter
    {
      get { return (IValueParameter<IntegerErcOptions>)Parameters[IntegerParameterName]; }
    }

    public IntegerErcOptions IntegerErcOptions
    {
      get { return this.IntegerErcOptionsParameter.Value; }
      set { this.IntegerErcOptionsParameter.Value = value; }
    }

    IReadOnlyIntegerErcOptions IReadOnlyErcOptions.IntegerErcOptions
    {
      get { return IntegerErcOptions; }
    }

    public IValueParameter<FloatErcOptions> FloatErcOptionsParameter
    {
      get { return (IValueParameter<FloatErcOptions>)Parameters[FloatParameterName]; }
    }

    public FloatErcOptions FloatErcOptions
    {
      get { return this.FloatErcOptionsParameter.Value; }
      set { this.FloatErcOptionsParameter.Value = value; }
    }

    IReadOnlyFloatErcOptions IReadOnlyErcOptions.FloatErcOptions
    {
      get { return FloatErcOptions; }
    }

    /// <summary>
    /// Integer represents ASCII value
    /// </summary>
    public IValueParameter<CharErcOptions> CharErcOptionsParameter
    {
      get { return (IValueParameter<CharErcOptions>)Parameters[CharParameterName]; }
    }

    public CharErcOptions CharErcOptions
    {
      get { return CharErcOptionsParameter.Value; }
      set { CharErcOptionsParameter.Value = value; }
    }

    IReadOnlyCharErcOptions IReadOnlyErcOptions.CharErcOptions
    {
      get { return CharErcOptions; }
    }

    public IValueParameter<BooleanErcOptions> BooleanErcOptionsParameter
    {
      get { return (IValueParameter<BooleanErcOptions>)Parameters[BooleanParameterName]; }
    }

    public BooleanErcOptions BooleanErcOptions
    {
      get { return BooleanErcOptionsParameter.Value; }
      set { BooleanErcOptionsParameter.Value = value; }
    }

    IReadOnlyBooleanErcOptions IReadOnlyErcOptions.BooleanErcOptions
    {
      get { return BooleanErcOptions; }
    }

    public IValueParameter<StringErcOptions> StringErcOptionsParameter
    {
      get { return (IValueParameter<StringErcOptions>)Parameters[StringParameterName]; }
    }

    public StringErcOptions StringErcOptions
    {
      get { return StringErcOptionsParameter.Value; }
      set { StringErcOptionsParameter.Value = value; }
    }

    IReadOnlyStringErcOptions IReadOnlyErcOptions.StringErcOptions
    {
      get { return StringErcOptions; }
    }

    public IValueParameter<NameErcOptions> NameParameter
    {
      get { return (IValueParameter<NameErcOptions>)Parameters[NameParameterName]; }
    }

    public NameErcOptions NameErcOptions
    {
      get { return NameParameter.Value; }
      set { NameParameter.Value = value; }
    }

    IReadOnlyNameErcOptions IReadOnlyErcOptions.NameErcOptions
    {
      get { return NameErcOptions; }
    }

    public IValueParameter<IntegerVectorErcOptions> IntegerVectorParameter
    {
      get { return (IValueParameter<IntegerVectorErcOptions>)Parameters[IntegerVectorParameterName]; }
    }

    public IntegerVectorErcOptions IntegerVectorErcOptions
    {
      get { return IntegerVectorParameter.Value; }
      set { IntegerVectorParameter.Value = value; }
    }

    IReadOnlyVectorErcOptions<int> IReadOnlyErcOptions.IntegerVectorErcOptions
    {
      get { return IntegerVectorErcOptions; }
    }

    public IValueParameter<StringVectorErcOptions> StringVectorParameter
    {
      get { return (IValueParameter<StringVectorErcOptions>)Parameters[StringVectorParameterName]; }
    }

    public StringVectorErcOptions StringVectorErcOptions
    {
      get { return StringVectorParameter.Value; }
      set { StringVectorParameter.Value = value; }
    }

    IReadOnlyVectorErcOptions<string> IReadOnlyErcOptions.StringVectorErcOptions
    {
      get { return StringVectorErcOptions; }
    }

    public IValueParameter<FloatVectorErcOptions> FloatVectorParameter
    {
      get { return (IValueParameter<FloatVectorErcOptions>)Parameters[FloatVectorParameterName]; }
    }

    public FloatVectorErcOptions FloatVectorErcOptions
    {
      get { return FloatVectorParameter.Value; }
      set { FloatVectorParameter.Value = value; }
    }

    IReadOnlyVectorErcOptions<double> IReadOnlyErcOptions.FloatVectorErcOptions
    {
      get { return FloatVectorErcOptions; }
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new ErcOptions(this, cloner);
    }
  }
}
