﻿using System.Linq;

namespace HeuristicLab.Problems.ProgramSynthesis.Base.Erc {
  using System.Collections.Generic;

  using Common;
  using Core;
  using Data;
  using Parameters;
  using Persistence.Default.CompositeSerializers.Storable;
  using Problems.ProgramSynthesis.Base.Erc.Interfaces;

  [StorableClass]
  public class IntegerVectorErcOptions : ErcOption, IReadOnlyVectorErcOptions<int> {
    private const string ConstantsParameterName = "Constants";

    public IntegerVectorErcOptions() : this(new int[0][]) {
    }

    public IntegerVectorErcOptions(int[][] constants) {
      IsEnabled = true;
      var mappedConstants = constants.Select(c => new IntArray(c));
      Parameters.Add(new ValueParameter<ItemArray<IntArray>>(ConstantsParameterName, new ItemArray<IntArray>(mappedConstants)));
    }

    [StorableConstructor]
    private IntegerVectorErcOptions(bool deserializing) : base(deserializing) { }

    public IntegerVectorErcOptions(IntegerVectorErcOptions origin, Cloner cloner) : base(origin, cloner) { }

    public IValueParameter<ItemArray<IntArray>> ConstantsParameter
    {
      get { return (IValueParameter<ItemArray<IntArray>>)Parameters[ConstantsParameterName]; }
    }

    public IEnumerable<IReadOnlyList<int>> Constants
    {
      get { return ConstantsParameter.Value.Select(v => v.Data); }
      set
      {
        var mappedConstants = value.Select(c => new IntArray(c.ToArray()));
        ConstantsParameter.Value = new ItemArray<IntArray>(mappedConstants);
      }
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new IntegerVectorErcOptions(this, cloner);
    }
  }
}
